#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <linux/videodev.h>

#include "sc.h"
#include "cc.h"

#define N_FRAMES	50
#define N_S_FRAMES	25

double calc_diff(unsigned char *cur, unsigned char *prev, int width, int height, int palette)
{
	char *rgb_cur  = convert_to_rgb(cur, width, height, palette);
	char *rgb_prev = convert_to_rgb(prev, width, height, palette);
	int loop, index=0;
	unsigned long int diff = 0;

	for(loop=0; loop<(width * height); loop++)
	{
		diff += abs(rgb_cur[index+0] - rgb_prev[index+0]) +
			abs(rgb_cur[index+1] - rgb_prev[index+1]) +
			abs(rgb_cur[index+2] - rgb_prev[index+2]);
		index += 3;
	}

	return ((double)diff) / ((double)(width * height * 3));
}

void usage(void)
{
	printf("Usage: measurenoise [video-device]\n");
	printf("Use an other tool to configure your video-device, for\n");
	printf("example 'dov4l'.\n");
}

int main(int argc, char *argv[])
{
	vconfig	vconf;
	char *device = "/dev/video0";
	double avg_noise = 0.0, min_noise, max_noise = 0.0;
	int loop;
	unsigned char *prev, *cur;
	int frame_size;

	printf("measurenoise, written by folkert@vanheusden.com\n\n");

	if (argc == 2)
		device = argv[1];
	else if (argc != 1)
	{
		usage();
		return 1;
	}

	printf("Using device: %s\n", device);

	/* open device */
	if (open_device(&vconf, device) == -1)
		return 2;

	/* init buffers */
	frame_size = vconf.vidmbuf.size / vconf.vidmbuf.frames;
	prev = (unsigned char *)malloc(frame_size);
	if (!prev)
	{
		fprintf(stderr, "Memory allocation error!\n\n");
		usage();
		return 3;
	}
	min_noise = frame_size * 256;

	/* let the device settle: take 25 pictures */
	for(loop=0; loop<N_S_FRAMES; loop++)
	{
		printf("Waiting for device to settle: %d   \r", N_S_FRAMES-loop);
		fflush(stdout);
		cur = take_picture(&vconf);
	}
	printf("\n");

	/* calculate! */
	for(loop=0; loop<N_FRAMES; loop++)
	{
		double cur_noise;

		printf("Measuring: %d   \r", N_FRAMES-loop);
		fflush(stdout);

		memcpy(prev, cur, frame_size);
		cur = take_picture(&vconf);

		cur_noise = calc_diff(cur, prev, vconf.vidwin.width, vconf.vidwin.height, vconf.vidpic.palette);

		min_noise = min_noise < cur_noise ? min_noise : cur_noise;
		max_noise = max_noise > cur_noise ? max_noise : cur_noise;
		avg_noise += cur_noise;
	}
	printf("\n");

	printf("Average: %.2f\n", avg_noise / (double)N_FRAMES);
	printf("Minimum: %.2f\n", min_noise);
	printf("Maximum: %.2f\n", max_noise);

	return 0;
}
